@@ -31,8 +31,11 @@ class Server(base_server.BaseServer):
     :param ping_timeout: The time in seconds that the client waits for the
                          server to respond before disconnecting. The default
                          is 20 seconds.
-    :param max_http_buffer_size: The maximum size of a message.  The default
-                                 is 1,000,000 bytes.
+    :param max_http_buffer_size: The maximum size that is accepted for incoming
+                                 messages.  The default is 1,000,000 bytes. In
+                                 spite of its name, the value set in this
+                                 argument is enforced for HTTP long-polling and
+                                 WebSocket connections.
     :param allow_upgrades: Whether to allow transport upgrades or not. The
                            default is ``True``.
     :param http_compression: Whether to compress packages when using the
@@ -306,7 +309,7 @@ class Server(base_server.BaseServer):
             r = self._method_not_found()
 
         if not isinstance(r, dict):
-            return r or []
+            return r
         if self.http_compression and \
                 len(r['response']) >= self.compression_threshold:
             encodings = [e.split(';')[0].strip() for e in
